Exception: ParseError
- Inherits:
-
StandardError
- Object
- StandardError
- ParseError
- Defined in:
- lib/rspec-interactive/rspec_core_parser.rb
Overview
In some cases abort is called in response to an exception. If we simply raise, the original exception will be logged as the cause. This will lead to duplicate messaging. Here we define our own exception so that we can ensure no cause is logged.
Instance Method Summary collapse
Instance Method Details
#cause ⇒ Object
11 12 13 |
# File 'lib/rspec-interactive/rspec_core_parser.rb', line 11 def cause nil end |